@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2013 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2014 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -80,6 +80,7 @@ int burn_setup_drive(struct burn_drive *d, char *fname)
 	d->do_stream_recording = 0;
         d->stream_recording_start= 0;
 	d->role_5_nwa = 0;
+	d->features = NULL;
 	return 1;
 }
 
@@ -104,6 +105,7 @@ void burn_drive_free_subs(struct burn_drive *d)
 	if (d->stdio_fd >= 0)
 		close (d->stdio_fd);
 	d->stdio_fd = -1;
+	burn_feature_descr_free(&(d->features), 0);
 	sg_dispose_drive(d, 0);
 }
 
@@ -261,8 +263,9 @@ int burn_drive_inquire_media(struct burn_drive *d)
 
         /* ts A71128 : run read_disc_info() for any recognizeable profile */
 	if (d->current_profile > 0 || d->current_is_guessed_profile ||
-	    d->mdata->cdr_write || d->mdata->cdrw_write ||
-	    d->mdata->dvdr_write || d->mdata->dvdram_write) {
+	    (d->mdata->p2a_valid > 0 &&
+		(d->mdata->cdr_write || d->mdata->cdrw_write ||
+		 d->mdata->dvdr_write || d->mdata->dvdram_write)) ) {
 
 #define Libburn_knows_correct_state_after_loaD 1
 #ifdef Libburn_knows_correct_state_after_loaD
@@ -390,7 +393,7 @@ static int burn_drive__is_rdwr(char *fname, int *stat_ret,
 		if (S_ISREG(stbuf.st_mode))
 			read_size = stbuf.st_size;
 		else if (is_rdwr)
-			ret = burn_os_stdio_capacity(fname, &read_size);
+			ret = burn_os_stdio_capacity(fname, 0, &read_size);
 		if (ret <= 0 ||
 		    read_size / (off_t) 2048 >= (off_t) 0x7ffffff0) 
 			read_size = (off_t) 0x7ffffff0 * (off_t) 2048;  
@@ -424,6 +427,8 @@ fprintf(stderr, "LIBBURN_DEBUG: burn_drive__is_rdwr: getfl_ret = %lX , O_RDWR =
 }
 
 
+/* flag bit0= ( not needed yet: grab even if it is already grabbed )
+*/
 int burn_drive_grab_stdio(struct burn_drive *d, int flag)
 {
 	int stat_ret = -1, is_rdwr, ret;
@@ -445,7 +450,7 @@ int burn_drive_grab_stdio(struct burn_drive *d, int flag)
 		d->media_read_capacity =
 					read_size / 2048 - !(read_size % 2048);
 		if ((stat_ret == -1 || is_rdwr) && d->devname[0]) { 
-		       	ret = burn_os_stdio_capacity(d->devname, &size);
+		       	ret = burn_os_stdio_capacity(d->devname, 0, &size);
 			if (ret > 0)
 				burn_drive_set_media_capacity_remaining(d,
 									size);
@@ -643,7 +648,7 @@ struct burn_drive *burn_drive_finish_enum(struct burn_drive *d)
 	        t->released = 1;
 	} else {
 		/* ts A90602 */
-		d->mdata->valid = -1;
+		d->mdata->p2a_valid = -1;
                 sprintf(msg, "Unable to grab scanned drive %s", d->devname);
                 libdax_msgs_submit(libdax_messenger, d->global_index,
                                 0x0002016f, LIBDAX_MSGS_SEV_DEBUG,
@@ -1216,6 +1221,8 @@ static void strip_spaces(char *str)
 static int drive_getcaps(struct burn_drive *d, struct burn_drive_info *out)
 {
 	struct burn_scsi_inquiry_data *id;
+	int i, profile;
+	struct burn_feature_descr *feat;
 
 	/* ts A61007 : now prevented in enumerate_common() */
 #if 0
@@ -1223,7 +1230,7 @@ static int drive_getcaps(struct burn_drive *d, struct burn_drive_info *out)
 	a ssert(d->mdata);
 #endif
 
-	if(d->idata->valid <= 0 || d->mdata->valid <= 0)
+	if(d->idata->valid <= 0)
 		return 0;
 
 	id = (struct burn_scsi_inquiry_data *)d->idata;
@@ -1236,18 +1243,48 @@ static int drive_getcaps(struct burn_drive *d, struct burn_drive_info *out)
 	strip_spaces(out->revision);
 	strncpy(out->location, d->devname, 16);
 	out->location[16] = '\0';
-	out->buffer_size = d->mdata->buffer_size;
-	out->read_dvdram = !!d->mdata->dvdram_read;
-	out->read_dvdr = !!d->mdata->dvdr_read;
-	out->read_dvdrom = !!d->mdata->dvdrom_read;
-	out->read_cdr = !!d->mdata->cdr_read;
-	out->read_cdrw = !!d->mdata->cdrw_read;
-	out->write_dvdram = !!d->mdata->dvdram_write;
-	out->write_dvdr = !!d->mdata->dvdr_write;
-	out->write_cdr = !!d->mdata->cdr_write;
-	out->write_cdrw = !!d->mdata->cdrw_write;
-	out->write_simulate = !!d->mdata->simulate;
-	out->c2_errors = !!d->mdata->c2_pointers;
+
+	if (d->mdata->p2a_valid > 0) {
+		out->buffer_size = d->mdata->buffer_size;
+		out->read_dvdram = !!d->mdata->dvdram_read;
+		out->read_dvdr = !!d->mdata->dvdr_read;
+		out->read_dvdrom = !!d->mdata->dvdrom_read;
+		out->read_cdr = !!d->mdata->cdr_read;
+		out->read_cdrw = !!d->mdata->cdrw_read;
+		out->write_dvdram = !!d->mdata->dvdram_write;
+		out->write_dvdr = !!d->mdata->dvdr_write;
+		out->write_cdr = !!d->mdata->cdr_write;
+		out->write_cdrw = !!d->mdata->cdrw_write;
+		out->write_simulate = !!d->mdata->simulate;
+		out->c2_errors = !!d->mdata->c2_pointers;
+	} else {
+		out->buffer_size = out->read_dvdram = out->read_dvdr = 0;
+		out->read_dvdrom = out->read_cdr = out->read_cdrw = 0;
+		out->write_dvdram = out->write_dvdr = out->write_cdr = 0;
+		out->write_cdrw = out->write_simulate = out->c2_errors = 0;
+		for (i = 0; i < d->num_profiles; i++) {
+			profile = (d->all_profiles[i * 4] << 8) |
+						d->all_profiles[i * 4 + 1];
+			if (profile == 0x09)
+				out->write_cdr = out->read_cdr = 1;
+			else if (profile == 0x0a)
+				out->write_cdrw = out->read_cdrw = 1;
+			else if (profile == 0x10)
+				out->read_dvdrom = 1;
+			else if (profile == 0x11)
+				out->write_dvdr = out->read_dvdr = 1;
+			else if (profile == 0x12)
+				out->write_dvdram = out->read_dvdram = 1;
+		}
+
+		/* Test Write bit of CD TAO, CD Mastering, DVD-R/-RW Write */
+		for (i = 0x002D; i <= 0x002F; i++)
+			if (burn_drive_has_feature(d, i, &feat, 0))
+				if (feat->data_lenght > 0)
+					out->write_simulate |=
+							 !!(feat->data[0] & 4);
+	}
+
 	out->drive = d;
 
 #ifdef Libburn_dummy_probe_write_modeS
@@ -1566,23 +1603,17 @@ void burn_sectors_to_msf(int sectors, int *m, int *s, int *f)
 
 int burn_drive_get_read_speed(struct burn_drive *d)
 {
-	if(d->mdata->valid <= 0)
-		return 0;
 	return d->mdata->max_read_speed;
 }
 
 int burn_drive_get_write_speed(struct burn_drive *d)
 {
-	if(d->mdata->valid <= 0)
-		return 0;
 	return d->mdata->max_write_speed;
 }
 
 /* ts A61021 : New API function */
 int burn_drive_get_min_write_speed(struct burn_drive *d)
 {
-	if(d->mdata->valid <= 0)
-		return 0;
 	return d->mdata->min_write_speed;
 }
 
@@ -1691,7 +1722,7 @@ int burn_drive_grab_dummy(struct burn_drive_info *drive_infos[], char *fname)
 		is_rdwr = burn_drive__is_rdwr(fname, &stat_ret, &stbuf,
 						&read_size, 1 | 2);
 		if (stat_ret == -1 || is_rdwr) {
-			ret = burn_os_stdio_capacity(fname, &size);
+			ret = burn_os_stdio_capacity(fname, 0, &size);
 			if (ret == -1) {
 				libdax_msgs_submit(libdax_messenger, -1,
 				 0x00020009,
@@ -2521,7 +2552,8 @@ int burn_disc_read_atip(struct burn_drive *d)
 	if(d->drive_role != 1)
 		return 0;
 	if ((d->current_profile == -1 || d->current_is_cd_profile)
-	    && (d->mdata->cdrw_write || d->current_profile != 0x08)) {
+	    && ((d->mdata->p2a_valid > 0 && d->mdata->cdrw_write) ||
+	        d->current_profile != 0x08)) {
 		d->read_atip(d);
 		/* >>> some control of success would be nice :) */
 	} else {
@@ -2605,7 +2637,8 @@ int burn_disc_get_msc1(struct burn_drive *d, int *start)
 off_t burn_disc_available_space(struct burn_drive *d,
 				 struct burn_write_opts *o)
 {
-	int lba, nwa;
+	int lba, nwa, ret;
+	off_t bytes, start_byte = 0;
 
 	if (burn_drive_is_released(d))
 		return 0;
@@ -2614,9 +2647,15 @@ off_t burn_disc_available_space(struct burn_drive *d,
 	if (d->drive_role == 0)
 		return 0;
 	if (d->drive_role != 1) {
-		if (d->media_capacity_remaining <= 0)
-			burn_drive_set_media_capacity_remaining(d,
-			  (off_t) (512 * 1024 * 1024 - 1) * (off_t) 2048);
+		if (o != NULL)
+			start_byte = o->start_byte;
+		ret = burn_os_stdio_capacity(d->devname, start_byte, &bytes);
+		if (ret != 1)
+			bytes = d->media_capacity_remaining;
+		if (bytes <= 0)
+			bytes = (off_t) (512 * 1024 * 1024 - 1) * (off_t) 2048;
+		if (bytes != d->media_capacity_remaining)
+			burn_drive_set_media_capacity_remaining(d, bytes);
 	} else {
 		if (o != NULL)
 			d->send_write_parameters(d, NULL, -1, o);
@@ -2750,8 +2789,6 @@ int burn_speed_descriptor_copy(struct burn_speed_descriptor *from,
 /* ts A61226 : free dynamically allocated sub data of struct scsi_mode_data */
 int burn_mdata_free_subs(struct scsi_mode_data *m)
 {
-	if(!m->valid)
-		return 0;
 	burn_speed_descriptor_destroy(&(m->speed_descriptors), 1);
 	return 1;
 }
@@ -2765,8 +2802,6 @@ int burn_drive_get_speedlist(struct burn_drive *d,
 	struct burn_speed_descriptor *sd, *csd = NULL;
 
 	(*speed_list) = NULL;
-	if(d->mdata->valid <= 0)
-		return 0;
 	for (sd = d->mdata->speed_descriptors; sd != NULL; sd = sd->next) {
 		ret = burn_speed_descriptor_new(&csd, NULL, csd, 0);
 		if (ret <= 0)
@@ -2790,8 +2825,6 @@ int burn_drive_get_best_speed(struct burn_drive *d, int speed_goal,
 	if (speed_goal < 0)
 		best_speed = 2000000000;
 	*best_descr = NULL;
-	if(d->mdata->valid <= 0)
-		return 0;
 	for (sd = d->mdata->speed_descriptors; sd != NULL; sd = sd->next) {
 		if (flag & 1)
 			speed = sd->read_speed;
@@ -2871,7 +2904,7 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 		/* stdio file drive : random access read-write */
 		o->start_adr = 1;
 		size = d->media_capacity_remaining;
-		burn_os_stdio_capacity(d->devname, &size);
+		burn_os_stdio_capacity(d->devname, 0, &size);
 		burn_drive_set_media_capacity_remaining(d, size);
 		o->start_range_high = d->media_capacity_remaining;
 		o->start_alignment = 2048; /* imposting a drive, not a file */
@@ -2883,7 +2916,7 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 		/* stdio file drive : random access write-only */
 		o->start_adr = 1;
 		size = d->media_capacity_remaining;
-		burn_os_stdio_capacity(d->devname, &size);
+		burn_os_stdio_capacity(d->devname, 0, &size);
 		burn_drive_set_media_capacity_remaining(d, size);
 
 		/* >>> start_range_low = file size rounded to 2048 */;
@@ -2935,7 +2968,8 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 			o->multi_session = o->multi_track = 0;
 		else if(wt == BURN_WRITE_NONE || wt == BURN_WRITE_SAO ||
 			wt == BURN_WRITE_TAO)
-			o->might_simulate = !!d->mdata->simulate;
+			o->might_simulate = !!(d->mdata->p2a_valid > 0 &&
+			                       d->mdata->simulate);
 	} else if (d->current_profile == 0x11 || d->current_profile == 0x14 ||
 			d->current_profile == 0x15) {
 		/* DVD-R , sequential DVD-RW , DVD-R/DL Sequential */
@@ -3430,3 +3464,70 @@ int burn_drive_was_feat21_failure(struct burn_drive *d)
 }
 
 
+/* ts B40106 */
+int burn_feature_descr_new(struct burn_feature_descr **new,
+                           unsigned char *descr, int descr_len, int flag)
+{
+	struct burn_feature_descr *o;
+
+	*new = NULL;
+	if (descr_len < 4)
+		return 0;
+	(*new) = o = calloc(1, sizeof(struct burn_speed_descriptor));
+	if (o == NULL)
+		return -1;
+	o->feature_code = (descr[0] << 8) | descr[1];
+        o->flags = descr[2];
+        if (descr[3] > descr_len - 4)
+		o->data_lenght = 0;
+	else
+        	o->data_lenght = descr[3];
+        o->data = NULL;
+        o->next = NULL;
+        if (o->data_lenght > 0) {
+		o->data = calloc(1, o->data_lenght);
+		if (o->data == NULL) {
+			burn_feature_descr_free(new, 0);
+			return -1;
+		}
+		memcpy(o->data, descr + 4, o->data_lenght);
+	}
+	return 1;
+}
+
+
+/* ts B40106 */
+int burn_feature_descr_free(struct burn_feature_descr **descr, int flag)
+{
+	struct burn_feature_descr *o, *next;
+
+	if (*descr == NULL)
+		return 0;
+	for (o = *descr; o != NULL; o = next) {
+		next = o->next;
+		if (o->data != NULL)
+			free(o->data);
+		free((char *) o);
+	}
+	*descr = NULL;
+	return 1;
+}
+
+
+/* ts B40107 */
+int burn_drive_has_feature(struct burn_drive *d, int feature_code,
+                           struct burn_feature_descr **descr, int flag)
+{
+	struct burn_feature_descr *o;
+
+	for (o = d->features; o != NULL; o = o->next) {
+		if (o->feature_code == feature_code) {
+			if (descr != NULL)
+				*descr = o;
+			return 1;
+		}
+	}
+	return 0;
+}
+
+
