@@ -1,5 +1,10 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
+/* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
+   Copyright (c) 2006 - 2010 Thomas Schmitt <scdbackup@gmx.net>
+   Provided under GPL version 2 or later.
+*/
+
 #ifndef BURN__FILE_H
 #define BURN__FILE_H
 
@@ -29,11 +34,13 @@ struct burn_source_fifo {
 	*/
 	int is_started;
 
+	void *thread_handle; /* actually a pointer to a thread_t */
 	int thread_pid;
-	int thread_pid_valid;
+	int thread_is_valid;
 
 	/* the burn_source for which this fifo is acting as proxy */
 	struct burn_source *inp;
+	int inp_read_size;
 
 	/* <<< up to now it was only a pipe. This is on its way out. */
 	int outlet[2];
@@ -50,6 +57,14 @@ struct burn_source_fifo {
 
 	off_t in_counter;
 	off_t out_counter;
+
+	int total_min_fill;
+	int interval_min_fill;
+	int put_counter;
+	int get_counter;
+	int empty_counter;
+	int full_counter;
+
 };
 
 
@@ -59,4 +74,26 @@ struct burn_source_fifo {
 int burn_fifo_source_shoveller(struct burn_source *source, int flag);
 
 
+/* ts B00922 */
+struct burn_source_offst {
+
+	/* See burn_offst_source_new() */
+	struct burn_source *inp;
+	struct burn_source *prev;
+	off_t start;
+	off_t size;
+	int size_adjustable;
+
+	/* for set_size/get_size */
+	int nominal_size;
+
+	/* To help offst_free() */
+	struct burn_source *next;
+
+	/* The current reading position */
+	int running;
+	off_t pos;
+
+};
+
 #endif /* LIBBURN__FILE_H */
